@@ -32,6 +32,7 @@ import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.Test;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 
@@ -120,6 +121,80 @@ public class ShutdownMonitorTest
         assertTrue(!monitor.isAlive());
     }
 
+    @Test
+    public void testNoExitSystemProperty() throws Exception
+    {
+        System.setProperty("STOP.EXIT", "false");
+        ShutdownMonitor monitor = ShutdownMonitor.getInstance();
+        monitor.setPort(0);
+        assertFalse(monitor.isExitVm());
+        monitor.start();
+
+        try (CloseableServer server = new CloseableServer())
+        {
+            server.setStopAtShutdown(true);
+            server.start();
+
+            //shouldn't be registered for shutdown on jvm
+            assertTrue(ShutdownThread.isRegistered(server));
+            assertTrue(ShutdownMonitor.isRegistered(server));
+
+            String key = monitor.getKey();
+            int port = monitor.getPort();
+
+            stop("stop", port, key, true);
+            monitor.await();
+
+            assertTrue(!monitor.isAlive());
+            assertTrue(server.stopped);
+            assertTrue(!server.destroyed);
+            assertTrue(!ShutdownThread.isRegistered(server));
+            assertTrue(!ShutdownMonitor.isRegistered(server));
+        }
+    }
+
+    /*
+     * Disable these config tests because ShutdownMonitor is a 
+     * static singleton that cannot be unset, and thus would
+     * need each of these methods executed it its own jvm -
+     * current surefire settings only fork for a single test 
+     * class.
+     * 
+     * Undisable to test individually as needed.
+     */
+    @Disabled
+    @Test
+    public void testExitVmDefault() throws Exception
+    {
+        //Test that the default is to exit
+        ShutdownMonitor monitor = ShutdownMonitor.getInstance();
+        monitor.setPort(0);
+        assertTrue(monitor.isExitVm());
+    }
+
+    @Disabled
+    @Test
+    public void testExitVmTrue() throws Exception
+    {
+        //Test setting exit true
+        System.setProperty("STOP.EXIT", "true");
+        ShutdownMonitor monitor = ShutdownMonitor.getInstance();
+        monitor.setPort(0);
+        assertTrue(monitor.isExitVm());
+    }
+    
+    @Disabled
+    @Test
+    public void testExitVmFalse() throws Exception
+    {
+        //Test setting exit false
+        System.setProperty("STOP.EXIT", "false");
+        ShutdownMonitor monitor = ShutdownMonitor.getInstance();
+        monitor.setPort(0);
+        assertFalse(monitor.isExitVm());
+    }
+    
+    @Disabled
     @Test
     public void testForceStopCommand() throws Exception
     {
