@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -21,9 +21,7 @@ package org.eclipse.jetty.start;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
-import java.io.InputStream;
 import java.io.OutputStream;
-import java.net.URL;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
@@ -40,6 +38,7 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
 import java.util.StringTokenizer;
+import java.util.jar.Attributes;
 import java.util.jar.Manifest;
 import java.util.stream.Collectors;
 import java.util.stream.StreamSupport;
@@ -66,61 +65,51 @@ public class StartArgs
     public static final Set<String> ARG_PARTS = Collections.unmodifiableSet(new HashSet<>(Arrays.asList(
         "args")));
 
+    private static final String JETTY_VERSION_KEY = "jetty.version";
+    private static final String JETTY_TAG_NAME_KEY = "jetty.tag.version";
+    private static final String JETTY_BUILDNUM_KEY = "jetty.build";
+
     static
     {
         // Use command line versions
-        String ver = System.getProperty("jetty.version", null);
-        String tag = System.getProperty("jetty.tag.version", "master");
+        String ver = System.getProperty(JETTY_VERSION_KEY);
+        String tag = System.getProperty(JETTY_TAG_NAME_KEY);
 
         // Use META-INF/MANIFEST.MF versions
         if (ver == null)
         {
             ver = ManifestUtils.getManifest(StartArgs.class)
                 .map(Manifest::getMainAttributes)
-                .filter(attributes -> "Eclipse Jetty Project".equals(attributes.getValue("Implementation-Vendor")))
-                .map(attributes -> attributes.getValue("Implementation-Version"))
+                .filter(attributes -> "Eclipse Jetty Project".equals(attributes.getValue(Attributes.Name.IMPLEMENTATION_VENDOR)))
+                .map(attributes -> attributes.getValue(Attributes.Name.IMPLEMENTATION_VERSION))
                 .orElse(null);
         }
 
-        // Use jetty-version.properties values
-        if (ver == null)
+        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
+        // use old jetty-version.properties (as seen within various linux distro repackaging of Jetty)
+        Props jettyVerProps = Props.load(classLoader, "jetty-version.properties");
+        // use build-time properties (included in start.jar) to pull version and buildNumber
+        Props buildProps = Props.load(classLoader, "org/eclipse/jetty/start/build.properties");
+
+        String sha = buildProps.getString("buildNumber", System.getProperty(JETTY_BUILDNUM_KEY));
+        if (Utils.isNotBlank(sha))
         {
-            URL url = Thread.currentThread().getContextClassLoader().getResource("jetty-version.properties");
-            if (url != null)
-            {
-                try (InputStream in = url.openStream())
-                {
-                    Properties props = new Properties();
-                    props.load(in);
-                    ver = props.getProperty("jetty.version");
-                }
-                catch (IOException x)
-                {
-                    StartLog.debug(x);
-                }
-            }
+            System.setProperty(JETTY_BUILDNUM_KEY, sha);
         }
 
-        // Default values
-        if (ver == null)
+        if (Utils.isBlank(ver))
         {
-            ver = "0.0";
-            if (tag == null)
-                tag = "master";
-        }
-        else
-        {
-            if (tag == null)
-                tag = "jetty-" + ver;
+            ver = jettyVerProps.getString("version", buildProps.getString("version", "0.0"));
         }
 
-        // Set Tag Defaults
-        if (tag.contains("-SNAPSHOT"))
-            tag = "master";
+        if (Utils.isBlank(tag))
+        {
+            tag = jettyVerProps.getString("tag", buildProps.getString("tag", "jetty-" + ver));
+        }
 
         VERSION = ver;
-        System.setProperty("jetty.version", VERSION);
-        System.setProperty("jetty.tag.version", tag);
+        System.setProperty(JETTY_VERSION_KEY, VERSION);
+        System.setProperty(JETTY_TAG_NAME_KEY, tag);
     }
 
     private static final String MAIN_CLASS = "org.eclipse.jetty.xml.XmlConfiguration";
@@ -323,8 +312,9 @@ public class StartArgs
         System.out.println();
         System.out.println("Jetty Environment:");
         System.out.println("-----------------");
-        dumpProperty("jetty.version");
-        dumpProperty("jetty.tag.version");
+        dumpProperty(JETTY_VERSION_KEY);
+        dumpProperty(JETTY_TAG_NAME_KEY);
+        dumpProperty(JETTY_BUILDNUM_KEY);
         dumpProperty("jetty.home");
         dumpProperty("jetty.base");
 
@@ -1513,6 +1503,7 @@ public class StartArgs
 
                 // ALPN feature exists
                 properties.setProperty("runtime.feature.alpn", Boolean.toString(isMethodAvailable(javax.net.ssl.SSLParameters.class, "getApplicationProtocols", null)), source);
+                properties.setProperty("at.least.java9", Boolean.toString(ver.getPlatform() >= 9), source);
             }
             catch (Throwable x)
             {
